@@ -285,6 +285,7 @@ class googleRoutesearchDownloader(Thread):
             res2 = self.downloadXml(self.CONFIG['google_routesearch_speak_street_lang'])
             self.parseXml( res2, self.CONFIG['google_routesearch_speak_street_lang'] )
 
+        self.geonames.recalculate_in_progress = 0
 
 class SaveTrackDialog( hildon.Program):
     def __init__(self, zoom=0, lat=0, lon=0):
@@ -327,6 +328,7 @@ class google_routesearch:
 
         self.icons = []
 
+        self.drive_mode = 0
 
         self.my_point = -1
 
@@ -338,6 +340,9 @@ class google_routesearch:
         self.espeak_dis = 0
         self.espeak_time = 0
         self.espeak_langs = []
+        
+        
+        self.recalculate_in_progress = 0
 
         if os.path.isfile("/usr/bin/espeak"):
             self.espeak = 1
@@ -377,6 +382,16 @@ class google_routesearch:
 
         #self.ico = "google_routesearch.png"
 
+        try:
+            print "google_routesearch auto recal:", self.CONFIG['google_routesearch_auto_recal']
+        except:
+            self.CONFIG['google_routesearch_auto_recal'] = 500
+        
+        try:
+            print "google_routesearch auto recal make:", self.CONFIG['google_routesearch_auto_recal_make']
+        except:
+            self.CONFIG['google_routesearch_auto_recal_make'] = 1
+
         try:
             print "google_routesearch last search:", self.CONFIG['google_routesearch_from']
         except:
@@ -386,11 +401,7 @@ class google_routesearch:
         except:
             self.CONFIG['google_routesearch_to'] = ""
 
-        try:
-            print "google_routesearch auto recalculate:", self.CONFIG['google_routesearch_auto_recalculate']
-        except:
-            self.CONFIG['google_routesearch_auto_recalculate'] = 1
-
+        
         try:
             print "google_routesearch mode:", self.CONFIG['google_routesearch_mode']
         except:
@@ -440,13 +451,13 @@ class google_routesearch:
 
         a = (pb[1]-pa[1])/(pb[0]-pa[0])
         c = pa[1]-( pa[0]*( a  ) )
-        b = (-pa[0]-c)/pa[1]
+        b = (((-a)*pa[0])-c)/pa[1]
 
-        d = pa[0]+pb[1]+c
+        d = a*pc[0]+b*pc[1]+c
         if d < 0:
             d = -d
         tr = d/( math.sqrt( (a*a)+(b*b) ) )
-
+        
         return tr
 
     def angle_from_vector(self, pa, pb, pc):
@@ -464,6 +475,8 @@ class google_routesearch:
         return res
 
     def do_draw(self, gpsmap, gdkdrawable):
+        print "drive mode", self.drive_mode
+        
         if self.osmMapa.osm.props.zoom>self.zoom_minimum:
 
 
@@ -472,7 +485,7 @@ class google_routesearch:
                     if p[3] <> None:
                         self.markers_icons.append( self.osmMapa.osm.image_add( p[0], p[1], p[3] ) )
 
-            if not self.CONFIG['google_routesearch_speak']:
+            if self.CONFIG['gpslock'] == 0:
 
                 t_lt, t_ln, b_lt, b_ln = self.osmMapa.osm.get_bbox()
                 t_lt, t_ln, b_lt, b_ln = math.degrees(t_lt), math.degrees(t_ln), math.degrees(b_lt), math.degrees(b_ln)
@@ -493,7 +506,7 @@ class google_routesearch:
             self.markers_icons = []
 
 
-        if self.CONFIG['google_routesearch_speak'] and len(self.points) > 0 and self.CONFIG['gpslock']:
+        if len(self.points) > 0 and self.recalculate_in_progress == 0 and self.CONFIG['gpslock']:
             print "calculate route and espeak"
             cr = gdkdrawable.cairo_create()
             # find close point
@@ -514,7 +527,7 @@ class google_routesearch:
                     angle = self.angle_from_vector( [p[0],p[1]], [latc,lonc],[p_old[0],p_old[1]] )
                     dis = self.osmMapa.distance( p[0],p[1], p_old[0],p_old[1] )
                     dis_total+=dis
-
+                    
                     if dis_min == -10:
                         dis_min = angle
                         id_next = i
@@ -531,6 +544,38 @@ class google_routesearch:
             dis_done = dis_total-dis_done
 
 
+            if self.CONFIG['google_routesearch_auto_recal_make']:
+                dis_recal = int(self.CONFIG['google_routesearch_auto_recal'])
+                a = self.points[id_next-1]
+                b = self.points[id_next]
+                c = [latc,lonc]
+
+                """
+                x,y = self.osmMapa.osm.geographic_to_screen(a[0],a[1])
+                self.osmMapa.draw_cloud( cr, x,y, ("a\n[%f,%f]"%(a[0],a[1])) )
+                
+                x,y = self.osmMapa.osm.geographic_to_screen(b[0],b[1])
+                self.osmMapa.draw_cloud( cr, x,y, ("b\n[%f,%f]"%(b[0],b[1])) )
+                
+                x,y = self.osmMapa.osm.geographic_to_screen(c[0],c[1])
+                self.osmMapa.draw_cloud( cr, x,y, ("c\n[%f,%f]"%(c[0],c[1])) )
+                """
+                
+                ll_ab = self.osmMapa.cal_dist( a[0],a[1],b[0],b[1])
+                km_ab = self.osmMapa.distance( a[0],a[1],b[0],b[1])
+                ll_cv = self.dist_from_vector(a,b,c)
+                
+                res_scale = ((ll_cv*km_ab)/ll_ab)*1000
+                
+                
+                print res_scale," max ",int(self.CONFIG['google_routesearch_auto_recal'])
+                if res_scale > int(self.CONFIG['google_routesearch_auto_recal']) and self.recalculate_in_progress == 0 and self.drive_mode == 1:
+                    self.recalculate_in_progress = 1
+                    self.clean_layer()
+                    self.CONFIG['google_routesearch_from'] = "my current location"
+                    self.search_action()
+                    
+
 
             search_for_it = -1
             for i in range(id_next, len(self.points)):
@@ -587,7 +632,8 @@ class google_routesearch:
                             self.espeak_time = t+5
 
                             speak = "%s. %s" % ( self.osmMapa.distance_to_string(dis_to_key),  m[2])
-                            self.espeak_cmd( speak )
+                            if self.CONFIG['google_routesearch_speak']:
+                                self.espeak_cmd( speak )
                         elif self.espeak_say == m[2] and t > (self.espeak_time):
                                 #print dis_to_key,"\t",self.espeak_dis,"\t",t
 
@@ -596,20 +642,23 @@ class google_routesearch:
                                     self.espeak_time = t+4
 
                                     speak = "%s. %s" % ( self.osmMapa.distance_to_string(dis_to_key),  m[2])
-                                    self.espeak_cmd( speak )
+                                    if self.CONFIG['google_routesearch_speak']:
+                                        self.espeak_cmd( speak )
 
                                 elif dis_to_key < 0.100 and self.espeak_dis > 0.100:
                                     self.espeak_dis = dis_to_key
                                     self.espeak_time = t+4
 
                                     speak = "%s. %s" % ( self.osmMapa.distance_to_string(dis_to_key),  m[2])
-                                    self.espeak_cmd( speak )
+                                    if self.CONFIG['google_routesearch_speak']:
+                                        self.espeak_cmd( speak )
                                 elif dis_to_key < 0.025 and self.espeak_dis > 0:
                                     self.espeak_dis = dis_to_key
                                     self.espeak_time = t+30
 
                                     speak = "%s" % ( m[2])
-                                    self.espeak_cmd( speak )
+                                    if self.CONFIG['google_routesearch_speak']:
+                                        self.espeak_cmd( speak )
 
 
 
@@ -636,6 +685,8 @@ class google_routesearch:
 
 
     def clean_layer(self):
+        self.drive_mode = 0
+        
         for i in self.markers_icons:
             self.osmMapa.osm.image_remove( i )
         self.markers_icons = []
@@ -661,7 +712,9 @@ class google_routesearch:
         dialog.destroy()
 
     def search_action(self, dialog=None):
+        self.recalculate_in_progress = 1
         self.clean_layer()
+        self.drive_mode = 1
 
         try:
             if self.bt_offline.get_active():
@@ -736,7 +789,21 @@ class google_routesearch:
                 break
 
         print "selected lang: ", self.CONFIG['google_routesearch_speak_street_lang']
+    
+    def on_changed_dis (self,picker,dialog):
+        str = picker.get_value()
+        for i in range( 100, 1500, 100):
+            if i == int(str):
+                self.CONFIG['google_routesearch_auto_recal'] = str
+                break
 
+        print "selected google_routesearch_auto_recal: ", self.CONFIG['google_routesearch_auto_recal']
+        
+    def on_bt_relac(self,w):
+        if w.get_active():
+            self.CONFIG['google_routesearch_auto_recal_make'] = 1
+        else:
+            self.CONFIG['google_routesearch_auto_recal_make'] = 0
 
     def on_bt_export(self,w, dialog):
         dialog.destroy()
@@ -809,28 +876,7 @@ class google_routesearch:
         vbox.pack_start( hbox, True )
 
 
-        if self.espeak:
-            self.bt_speak = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
-            self.bt_speak.set_label("speak")
-            if self.CONFIG['google_routesearch_speak']:
-                self.bt_speak.set_active( True )
-            self.bt_speak.connect("clicked", self.on_bt_speak)
-            hbox.pack_start(self.bt_speak, True)
-
-
-            pb_lang = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
-                                        hildon.BUTTON_ARRANGEMENT_VERTICAL)
-            pb_lang.set_title("lang")
-            selector_lang = hildon.TouchSelectorEntry(text=True)
-            id = 0
-            for i,lang_name in enumerate(self.espeak_langs):
-                selector_lang.append_text(lang_name)
-                if lang_name == self.CONFIG['google_routesearch_speak_street_lang']:
-                    id = i
-            pb_lang.set_selector(selector_lang)
-            pb_lang.set_active( id  )
-            pb_lang.connect("value-changed",self.on_changed_lang, dialog)
-            hbox.pack_start( pb_lang, True )
+        
 
 
         """
@@ -870,10 +916,80 @@ class google_routesearch:
         vbox.pack_start(hbox, True)
 
 
+                
+        extra = gtk.Expander()
+        extra.set_label("Extras:")
+        ve = gtk.VBox()
+        extra.add(ve)
+        
+        if self.espeak:
+            l = gtk.Label("espeak settings:")
+            ve.pack_start(l,False)
+            hbox = gtk.HBox()
+            self.bt_speak = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
+            self.bt_speak.set_label("speak")
+            if self.CONFIG['google_routesearch_speak']:
+                self.bt_speak.set_active( True )
+            self.bt_speak.connect("clicked", self.on_bt_speak)
+            hbox.pack_start(self.bt_speak, True)
+
+
+            pb_lang = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
+                                        hildon.BUTTON_ARRANGEMENT_VERTICAL)
+            pb_lang.set_title("lang")
+            selector_lang = hildon.TouchSelectorEntry(text=True)
+            id = 0
+            for i,lang_name in enumerate(self.espeak_langs):
+                selector_lang.append_text(lang_name)
+                if lang_name == self.CONFIG['google_routesearch_speak_street_lang']:
+                    id = i
+            pb_lang.set_selector(selector_lang)
+            pb_lang.set_active( id  )
+            pb_lang.connect("value-changed",self.on_changed_lang, dialog)
+            hbox.pack_start( pb_lang, True )
+            ve.pack_start(hbox,True)
+            ve.pack_start(gtk.HSeparator(),True)
+        
         if len( self.tracks ):
             bt_export = gtk.Button("Export route to gpx")
             bt_export.connect("clicked", self.on_bt_export, dialog )
-            vbox.pack_start( bt_export, True)
+            ve.pack_start( bt_export, True)
+            ve.pack_start(gtk.HSeparator(),True)
+        
+        
+        l = gtk.Label("auto recalculation:")
+        ve.pack_start(l,False)
+        hbox = gtk.HBox()
+        #self.CONFIG['google_routesearch_auto_recal']
+        
+        self.bt_recal = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
+        self.bt_recal.set_label("make it")
+        self.bt_recal.set_active( self.CONFIG['google_routesearch_auto_recal_make'] )
+        self.bt_recal.connect("clicked", self.on_bt_relac )
+        hbox.pack_start(self.bt_recal, True)
+                
+        pb_dis = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
+                                        hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        pb_dis.set_title("if distance is greater than [meters]")
+        selector_dis = hildon.TouchSelectorEntry(text=True)
+        id = 0
+        for i in range( 100, 1600, 100 ):
+            selector_dis.append_text(("%s"%i))
+            if i == int(self.CONFIG['google_routesearch_auto_recal']):
+                print "set_active",i
+                id = i
+        pb_dis.set_selector(selector_dis)
+        pb_dis.set_active( id  )
+        pb_dis.connect("value-changed",self.on_changed_dis, dialog)
+        hbox.pack_start( pb_dis, True )
+        
+        
+        ve.pack_start(hbox,True)
+        ve.pack_start(gtk.HSeparator(),True)
+        
+        
+        
+        vbox.pack_start( extra, True)
 
         return vbox
 
