@@ -63,6 +63,10 @@ gtk.gdk.threads_init()
 name = "yosmapa"
 version = "0.19"
 """
+0.19-4
+    plugin[google_routesearch] trip predictions for non speak mode
+        clean up widget
+        add auto recalculation
 0.19-3
     plugin[google_routesearch] trip predictions
 0.19-2
